/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * @file stat.c
 * Implementation of the s3 DSI stat
 */

#include "globus_gridftp_server_s3_stat.h"

static
S3Status
globus_l_s3_stat_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg);

static
void
globus_l_s3_stat_complete(
    S3Status                            status,
    const S3ErrorDetails               *errorDetails,
    void                               *callbackData);

static
S3Status
globus_l_s3_stat_list_service(
    const char                         *ownerId, 
    const char                         *ownerDisplayName,
    const char                         *bucketName,
    int64_t                             creationDateSeconds,
    void                               *callbackData);


void
globus_i_s3_stat_list_buckets(
    globus_i_s3_stat_t                 *stat_state)
{
    stat_state->retries = 0;

    do
    {
        stat_state->status = S3StatusOK;
        stat_state->result = GLOBUS_SUCCESS;

        globus_i_s3_retry_delay(stat_state->retries);

        GlobusS3Info("REST: op=%p S3_list_service() try=%d\n",
                    (void *) stat_state->op,
                    stat_state->retries);

        S3_list_service(
                stat_state->bucket_context.protocol,
                stat_state->bucket_context.accessKeyId,
                stat_state->bucket_context.secretAccessKey,
                stat_state->bucket_context.securityToken,
                NULL,
                NULL,
                &(S3ListServiceHandler)
                {
                    .responseHandler      = 
                    {
                        .propertiesCallback
                                          = globus_l_s3_stat_properties,
                        .completeCallback = globus_l_s3_stat_complete
                    },
                    .listServiceCallback  = globus_l_s3_stat_list_service
                },
                stat_state);

        GlobusS3DebugOpStatus(S3_list_service, stat_state);
    }
    while (S3_status_is_retryable(stat_state->status)
        && (stat_state->retries++) < globus_i_s3_max_retries);

    return;
}
/* globus_l_s3_stat_list_buckets() */

static
S3Status
globus_l_s3_stat_properties(
    const S3ResponseProperties         *properties,
    void                               *callback_arg)
{
    S3Status                            status = S3StatusOK;

    GlobusS3Enter();

    GlobusS3ExitStatus(status);
    return status;
}
/* globus_l_s3_stat_properties() */

static
void
globus_l_s3_stat_complete(
    S3Status                            status,
    const S3ErrorDetails               *error_details,
    void                               *callback_data)
{
    globus_i_s3_stat_t                 *stat_state = callback_data;

    GlobusS3Enter();

    if (status != S3StatusOK && stat_state->result == GLOBUS_SUCCESS)
    {
        stat_state->status = status;
        stat_state->result = GlobusGFSErrorS3(
                    status,
                    stat_state->stat_info->pathname,
                    error_details);
    }
    GlobusS3Exit();
}
/* globus_l_s3_stat_complete() */

static
S3Status
globus_l_s3_stat_list_service(
    const char                         *ownerId, 
    const char                         *owner_display_name,
    const char                         *bucket_name,
    int64_t                             creation_date_seconds,
    void                               *callback_data)
{
    globus_i_s3_stat_t                 *stat_state = callback_data;
    S3Status                            status = S3StatusOK;
    int                                 dev;
    int                                 ino;

    GlobusS3Enter();

    dev = globus_hashtable_string_hash((void *) bucket_name, INT_MAX);
    ino = globus_hashtable_string_hash((void *) bucket_name, INT_MAX);

    globus_i_s3_stat_add(
            stat_state,
            &(globus_gfs_stat_t)
            {
                .mode                 = S_IRWXU|S_IFDIR,
                .nlink                = 1,
                .name                 = (char *) bucket_name,
                .size                 = 0,
                .mtime                = (time_t) -1,
                .ctime                = (time_t) creation_date_seconds,
                .atime                = (time_t) -1,
                .dev                  = dev,
                .ino                  = ino
            });

    GlobusS3ExitStatus(status);
    return status;
}
/* globus_l_s3_stat_list_service() */
// vim: ft=c:
